@@ -1,23 +1,4 @@
-#!/usr/bin/env python
-
-# Copyright (C) 2003-2007  Robey Pointer <robeypointer@gmail.com>
-#
-# This file is part of paramiko.
-#
-# Paramiko is free software; you can redistribute it and/or modify it under the
-# terms of the GNU Lesser General Public License as published by the Free
-# Software Foundation; either version 2.1 of the License, or (at your option)
-# any later version.
-#
-# Paramiko is distributed in the hope that it will be useful, but WITHOUT ANY
-# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
-# A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more
-# details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with Paramiko; if not, write to the Free Software Foundation, Inc.,
-# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.
-
+import base64
 import random
 import socket
 # import sys
@@ -26,7 +7,6 @@ import threading
 import paramiko
 
 from binascii import hexlify
-from paramiko.py3compat import u, decodebytes
 from tests.utils import make_tests_data_path
 
 
@@ -36,7 +16,7 @@ paramiko.util.log_to_file(make_tests_data_path('sshserver.log'))
 host_key = paramiko.RSAKey(filename=make_tests_data_path('test_rsa.key'))
 # host_key = paramiko.DSSKey(filename='test_dss.key')
 
-print('Read key: ' + u(hexlify(host_key.get_fingerprint())))
+print('Read key: ' + hexlify(host_key.get_fingerprint()).decode('utf-8'))
 
 banner = u'\r\n\u6b22\u8fce\r\n'
 event_timeout = 5
@@ -49,17 +29,27 @@ class Server(paramiko.ServerInterface):
             b'fAu7jJ2d7eothvfeuoRFtJwhUmZDluRdFyhFY/hFAh76PJKGAusIqIQKlkJxMC'
             b'KDqIexkgHAfID/6mqvmnSJf0b5W8v5h2pI/stOSwTQ+pxVhwJ9ctYDhRSlF0iT'
             b'UWT10hcuO4Ks8=')
-    good_pub_key = paramiko.RSAKey(data=decodebytes(data))
+    good_pub_key = paramiko.RSAKey(data=base64.decodebytes(data))
 
+    commands = [
+        b'$SHELL -ilc "locale charmap"',
+        b'$SHELL -ic "locale charmap"'
+    ]
     encodings = ['UTF-8', 'GBK', 'UTF-8\r\n', 'GBK\r\n']
 
-    def __init__(self):
+    def __init__(self, encodings=[]):
         self.shell_event = threading.Event()
         self.exec_event = threading.Event()
-        self.encoding = random.choice(self.encodings)
+        self.cmd_to_enc = self.get_cmd2enc(encodings)
         self.password_verified = False
         self.key_verified = False
 
+    def get_cmd2enc(self, encodings):
+        n = len(self.commands)
+        while len(encodings) < n:
+            encodings.append(random.choice(self.encodings))
+        return dict(zip(self.commands, encodings[0:n]))
+
     def check_channel_request(self, kind, chanid):
         if kind == 'session':
             return paramiko.OPEN_SUCCEEDED
@@ -72,7 +62,7 @@ class Server(paramiko.ServerInterface):
         return paramiko.AUTH_FAILED
 
     def check_auth_publickey(self, username, key):
-        print('Auth attempt with username: {!r} & key: {!r}'.format(username, u(hexlify(key.get_fingerprint())))) # noqa
+        print('Auth attempt with username: {!r} & key: {!r}'.format(username, hexlify(key.get_fingerprint()).decode('utf-8'))) # noqa
         if (username in ['robey', 'keyonly']) and (key == self.good_pub_key):
             return paramiko.AUTH_SUCCESSFUL
         if username == 'pkey2fa' and key == self.good_pub_key:
@@ -122,10 +112,11 @@ class Server(paramiko.ServerInterface):
         return 'password,publickey'
 
     def check_channel_exec_request(self, channel, command):
-        if command != b'locale charmap':
+        if command not in self.commands:
             ret = False
         else:
             ret = True
+            self.encoding = self.cmd_to_enc[command]
             channel.send(self.encoding)
             channel.shutdown(1)
         self.exec_event.set()
@@ -145,7 +136,7 @@ class Server(paramiko.ServerInterface):
         return True
 
 
-def run_ssh_server(port=2200, running=True):
+def run_ssh_server(port=2200, running=True, encodings=[]):
     # now connect
     sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
@@ -159,7 +150,7 @@ def run_ssh_server(port=2200, running=True):
         t = paramiko.Transport(client)
         t.load_server_moduli()
         t.add_server_key(host_key)
-        server = Server()
+        server = Server(encodings)
         try:
             t.start_server(server=server)
         except Exception as e:
@@ -187,7 +178,12 @@ def run_ssh_server(port=2200, running=True):
 
         # chan.send('\r\n\r\nWelcome!\r\n\r\n')
         print(server.encoding)
-        chan.send(banner.encode(server.encoding.strip()))
+        try:
+            banner_encoded = banner.encode(server.encoding)
+        except (ValueError, LookupError):
+            continue
+
+        chan.send(banner_encoded)
         if username == 'bar':
             msg = chan.recv(1024)
             chan.send(msg)
